@@ -2,14 +2,14 @@
 #  | | | | / __| / _ \ / __|| '__| / _ \ / _` || '_ ` _ \
 #  | |_| || (__ |  __/| (__ | |   |  __/| (_| || | | | | |
 #   \__, | \___| \___| \___||_|    \___| \__,_||_| |_| |_|
-#   |___/     the no depencency, Pythonic fork of IceCream
+#   |___/                          makes debugging sweeter
 #
-#  See https://raw.githubusercontent.com/salabim/ycecream/master/readme.md for details
+#      See https://github.com/salabim/ycecream for details
 
-__version__ = "1.1.0"
+__version__ = "1.1.4"
 
 """
-Fork from IceCream - Never use print() to debug again
+Fork of IceCream - Never use print() to debug again
 Original author: Ansgar Grunseid / grunseid.com / grunseid@gmail.com
 
 (c)2021 Ruud van der Ham - rt.van.der.ham@gmail.com
@@ -19,14 +19,57 @@ import ast
 import inspect
 import sys
 import datetime
+import time
 import textwrap
 from pathlib import Path
 from contextlib import contextmanager
 from functools import wraps
+import json
+import logging
+
+YCECREAM = "ycecream"
 
 
-def stderr_print(*args):
-    print(*args, file=sys.stderr)
+class default:
+    pass
+
+
+def set_defaults():
+    default.prefix = "y| "
+    default.output = "stderr"
+    default.serialize = lambda obj, sort_dicts=False: pformat(obj, sort_dicts=sort_dicts).replace("\\n", "\n")
+    default.show_context = False
+    default.show_time = False
+    default.show_delta = False
+    default.sort_dicts = False
+    default.show_enter = True
+    default.show_exit = True
+    default.enabled = True
+    default.line_length = 80
+    default.context_delimiter = " ==> "
+    default.pair_delimiter = ", "
+
+    config = {}
+    for path in sys.path:
+        path = Path(path)
+        if (path / f"{YCECREAM}.json").is_file():
+            with open(path / f"{YCECREAM}.json", "r") as f:
+                config = json.load(f)
+            break
+        if (path / "YCECREAM" / f"{YCECREAM}.json").is_file():
+            with open(path / {YCECREAM} / f"{YCECREAM}.json", "r") as f:
+                config = json.load(f)
+            break
+
+    for k, v in config.items():
+        if k == "serialize":
+            raise ValueError(f"error in {path}: key {k} not allowed")
+
+        elif hasattr(default, k):
+            setattr(default, k, v)
+
+        else:
+            raise ValueError(f"error in {path}: key {k} not recognized")
 
 
 def isLiteral(s):
@@ -76,49 +119,49 @@ def format_pair(prefix, arg, value):
     lines = arg_lines[:-1] + value_lines
     return "\n".join(lines)
 
+
 def check_output(output):
     if callable(output):
         return output
     if isinstance(output, (str, Path)):
         return output
     try:
-        output.write('')
+        output.write("")
         return output
     except Exception:
         pass
-    raise ValueError("output is neither a callable, str, Path or open text file")    
+    raise ValueError("output should be a callable, str, Path or open text file.")
+
 
 def do_output(output, s):
     if callable(output):
         output(s)
-        return
-    if isinstance(output, (str, Path)):
-        if output != '':
-            with open(output, "a+") as f:
-                print(s, file=f)
-        return
-    print(s, file=output)
+    elif output == "stderr":
+        print(s, file=sys.stderr)
+    elif output == "stdout":
+        print(s, file=sys.stdout)
+    elif output == "logging.debug":
+        logging.debug(s)
+    elif output == "logging.info":
+        logging.info(s)
+    elif output == "logging.warning":
+        logging.warning(s)
+    elif output == "logging.error":
+        logging.error(s)
+    elif output == "logging.critical":
+        logging.critical(s)
+    elif output in ("", "null"):
+        pass
+
+    elif isinstance(output, (str, Path)):
+        with open(output, "a+", encoding="utf-8") as f:
+            print(s, file=f)
+    else:
+        print(s, file=output)
 
 
-
-PREFIX = "y| "
-LINE_LENGTH = 80  # Characters.
-CONTEXT_DELIMITER = " ==> "
-OUTPUT = lambda *args: stderr_print(*args)
-SERIALIZE = lambda obj, sort_dicts=False: pformat(obj, sort_dicts=sort_dicts).replace("\\n", "\n")
-SHOW_CONTEXT = False
-SHOW_TIME = False
-SHOW_DELTA = False
-PAIR_DELIMITER = ", "
-ENABLED = True
-SORT_DICTS = False
-AS_STR = False
-SHOW_ENTER = True
-SHOW_EXIT = True
 global_enabled = True
 
-starttime = datetime.datetime.now()
-
 
 class Y:
     def __init__(
@@ -130,28 +173,29 @@ class Y:
         show_context=None,
         show_time=None,
         show_delta=None,
-        line_length=None,
-        pair_delimiter=None,
-        enabled=None,
-        sort_dicts=None,
-        as_str=None,
         show_enter=None,
         show_exit=None,
+        sort_dicts=None,
+        enabled=None,
+        line_length=None,
+        context_delimiter=None,
+        pair_delimiter=None
     ):
 
-        self.prefix = PREFIX if prefix is None else prefix
-        self.output = OUTPUT if output is None else check_output(output)
-        self.serialize = SERIALIZE if serialize is None else serialize
-        self.show_context = SHOW_CONTEXT if show_context is None else show_context
-        self.show_time = SHOW_TIME if show_time is None else show_time
-        self.show_delta = SHOW_DELTA if show_delta is None else show_delta
-        self.line_length = LINE_LENGTH if line_length is None else line_length
-        self.pair_delimiter = PAIR_DELIMITER if pair_delimiter is None else pair_delimiter
-        self.enabled = ENABLED if enabled is None else enabled
-        self.sort_dicts = SORT_DICTS if sort_dicts is None else sort_dicts
-        self.as_str = AS_STR if as_str is None else as_str
-        self.show_enter = SHOW_ENTER if show_enter is None else show_enter
-        self.show_exit = SHOW_EXIT if show_exit is None else show_exit
+        self.prefix = default.prefix if prefix is None else prefix
+        self.output = default.output if output is None else check_output(output)
+        self.serialize = default.serialize if serialize is None else serialize
+        self.show_context = default.show_context if show_context is None else show_context
+        self.show_time = default.show_time if show_time is None else show_time
+        self.show_delta = default.show_delta if show_delta is None else show_delta
+        self.show_enter = default.show_enter if show_enter is None else show_enter
+        self.show_exit = default.show_exit if show_exit is None else show_exit
+        self.sort_dicts = default.sort_dicts if sort_dicts is None else sort_dicts
+        self.enabled = default.enabled if enabled is None else enabled
+        self.line_length = default.line_length if line_length is None else line_length
+        self.context_delimiter = default.context_delimiter if context_delimiter is None else context_delimiter
+        self.pair_delimiter = default.pair_delimiter if pair_delimiter is None else pair_delimiter
+        self.start_time = time.perf_counter()
 
     def __call__(
         self,
@@ -162,64 +206,84 @@ class Y:
         show_context=None,
         show_time=None,
         show_delta=None,
-        line_length=None,
-        pair_delimiter=None,
-        enabled=None,
-        sort_dicts=None,
-        as_str=None,
         show_enter=None,
-        show_exit=None
+        show_exit=None,
+        sort_dicts=None,
+        enabled=None,
+        line_length=None,
+        context_delimiter=None,
+        pair_delimiter=None,
+        as_str=False
     ):
         call_frame = inspect.currentframe().f_back
         frame = inspect.getframeinfo(call_frame, context=1)
+        this_line = frame.code_context[0].strip()
 
-        if frame.code_context[0].strip().startswith("@"):
+        if this_line.startswith("@"):
             prefix = self.prefix if prefix is None else prefix
             output = self.output if output is None else check_output(output)
             serialize = self.serialize if serialize is None else serialize
             show_context = self.show_context if show_context is None else show_context
             show_time = self.show_time if show_time is None else show_time
             show_delta = self.show_delta if show_delta is None else show_delta
-            line_length = self.line_length if line_length is None else line_length
-            pair_delimiter = self.pair_delimiter if pair_delimiter is None else pair_delimiter
-            enabled = self.enabled if enabled is None else enabled
-            sort_dicts = self.sort_dicts if sort_dicts is None else sort_dicts
             show_enter = self.show_enter if show_enter is None else show_enter
             show_exit = self.show_exit if show_exit is None else show_exit
- 
+            sort_dicts = self.sort_dicts if sort_dicts is None else sort_dicts
+            enabled = self.enabled if enabled is None else enabled
+            line_length = self.line_length if line_length is None else line_length
+            context_delimiter = self.context_delimiter if context_delimiter is None else context_delimiter
+            pair_delimiter = self.pair_delimiter if pair_delimiter is None else pair_delimiter
+            if as_str:
+                raise TypeError("as_str may not be True when y used as decorator")
             frame_info = inspect.getframeinfo(call_frame)
             filename = Path(frame_info.filename).name
-            line_number = frame_info.lineno
+            line_number = frame_info.lineno + 1
             parent_function = frame_info.function
             context_info = f"{filename}:{line_number} in {parent_function}"
 
             def real_decorator(function):
                 @wraps(function)
                 def wrapper(*args, **kwargs):
+                    enter_time = time.perf_counter()
                     if show_context:
                         parts = [context_info]
                     else:
                         parts = []
                     if show_time:
-                        parts.append(f'@ {datetime.datetime.now().strftime("%H:%M:%S.%f")[:-3]}')
+                        parts.append(f'@ {datetime.datetime.now().strftime("%H:%M:%S.%f")}')
 
                     if show_delta:
-                        t0 = (datetime.datetime.now() - starttime).total_seconds()
-                        parts.append(f"\u0394 {t0:.3f}")
+                        t0 = time.perf_counter() - self.start_time
+                        parts.append(f"delta={t0:.3f}")
 
                     context = " ".join(parts)
                     if context:
-                        context += CONTEXT_DELIMITER
+                        context += context_delimiter
 
                     args_kwargs = [repr(arg) for arg in args] + [f"{str(k)}={repr(v)}" for k, v in kwargs.items()]
                     function_arguments = f"{function.__name__}({', '.join(args_kwargs)})"
 
                     if global_enabled and enabled and show_enter:
-                        do_output(output,f"{prefix() if callable(prefix) else prefix}{context}called {function_arguments}")
-
+                        do_output(output, f"{prefix() if callable(prefix) else prefix}{context}called {function_arguments}")
                     result = function(*args, **kwargs)
+                    duration = time.perf_counter() - enter_time
                     if global_enabled and enabled and show_exit:
-                        do_output(output,f"{call_or_value(prefix)}{context}returned {repr(result)} from {function_arguments}")
+                        if show_context:
+                            parts = [context_info]
+                        else:
+                            parts = []
+                        if show_time:
+                            parts.append(f'@ {datetime.datetime.now().strftime("%H:%M:%S.%f")}')
+
+                        if show_delta:
+                            t0 = time.perf_counter() - self.start_time
+                            parts.append(f"delta={t0:.3f}")
+
+                        context = " ".join(parts)
+                        if context:
+                            context += context_delimiter
+
+                        do_output(output, f"{call_or_value(prefix)}{context}returned {repr(result)} from {function_arguments} in {duration:.6f} seconds")
                     return result
 
                 return wrapper
@@ -229,7 +293,7 @@ class Y:
 
             if len(args) == 1 and callable(args[0]):
                 return real_decorator(args[0])
-            raise ValueError("arguments are not allowed in y used as decorator")
+            raise TypeError("arguments are not allowed in y used as decorator")
             return  # not really required
 
         self._prefix = self.prefix if prefix is None else prefix
@@ -238,21 +302,51 @@ class Y:
         self._show_context = self.show_context if show_context is None else show_context
         self._show_time = self.show_time if show_time is None else show_time
         self._show_delta = self.show_delta if show_delta is None else show_delta
+        self._sort_dicts = self.sort_dicts if sort_dicts is None else sort_dicts
+        self._as_str = as_str
+        self._enabled = self.enabled if enabled is None else enabled
         self._line_length = self.line_length if line_length is None else line_length
         self._pair_delimiter = self.pair_delimiter if pair_delimiter is None else pair_delimiter
-        self._enabled = self.enabled if enabled is None else enabled
-        self._sort_dicts = self.sort_dicts if sort_dicts is None else sort_dicts
-        self._as_str = self.as_str if as_str is None else as_str       
-        
+        self._context_delimiter = self.context_delimiter if context_delimiter is None else context_delimiter
+        if this_line.startswith("with ") or this_line.startswith("with\t"):
+            if as_str:
+                raise TypeError("as_str may not be True when y used as context manager")
+            if args:
+                raise TypeError("non-keyword arguments are not allowed when y used as context manager")
+
+            frame_info = inspect.getframeinfo(call_frame)
+            filename = Path(frame_info.filename).name
+            line_number = frame_info.lineno
+            context_info = f"{filename}:{line_number}"
+
+            self._show_enter = self.show_enter if show_enter is None else show_enter
+            self._show_exit = self.show_exit if show_exit is None else show_exit
+            new_y = self.clone(
+                prefix=prefix,
+                output=output,
+                serialize=serialize,
+                show_context=show_context,
+                show_time=show_time,
+                show_delta=show_delta,
+                show_enter=show_enter,
+                show_exit=show_exit,
+                sort_dicts=sort_dicts,
+                enabled=enabled,
+                context_delimiter=context_delimiter,
+                pair_delimiter=pair_delimiter,
+                line_length=line_length,
+            )
+            new_y.cm_info = context_info
+            return new_y
         try:
             out = self._format(call_frame, *args)
         except NoSourceAvailableError as err:
             prefix = call_or_value(self._prefix)
             out = prefix + "Error: " + err.fail_message
         if self._as_str:
-            return out
+            return out + "\n"
         if global_enabled and self._enabled:
-            do_output(self._output,out)
+            do_output(self._output, out)
 
         if len(args) == 0:
             passthrough = None
@@ -272,13 +366,13 @@ class Y:
         show_context=None,
         show_time=None,
         show_delta=None,
-        line_length=None,
-        pair_delimiter=None,
-        enabled=None,
-        sort_dicts=None,
-        as_str=False,
         show_enter=None,
-        show_exit=None
+        show_exit=None,
+        sort_dicts=None,
+        enabled=None,
+        line_length=None,
+        context_delimiter=None,
+        pair_delimiter=None
     ):
         self.prefix = self.prefix if prefix is None else prefix
         self.output = self.output if output is None else check_output(output)
@@ -286,13 +380,14 @@ class Y:
         self.show_context = self.show_context if show_context is None else show_context
         self.show_time = self.show_time if show_time is None else show_time
         self.show_delta = self.show_delta if show_delta is None else show_delta
-        self.line_length = self.line_length if line_length is None else line_length
-        self.pair_delimiter = self.pair_delimiter if pair_delimiter is None else pair_delimiter
-        self.enabled = self.enabled if enabled is None else enabled
-        self.sort_dicts = self.sort_dicts if sort_dicts is None else sort_dicts
-        self.as_str = self.as_str if as_str is None else as_str
         self.show_enter = self.show_enter if show_enter is None else show_enter
-        self.decoraror_exit = self.show_exit if show_exit is None else show_exit
+        self.show_exit = self.show_exit if show_exit is None else show_exit
+        self.sort_dicts = self.sort_dicts if sort_dicts is None else sort_dicts
+        self.enabled = self.enabled if enabled is None else enabled
+        self.line_length = self.line_length if line_length is None else line_length
+        self.context_delimiter = self.context_delimiter if context_delimiter is None else context_delimiter
+        self.pair_delimiter = self.pair_delimiter if pair_delimiter is None else pair_delimiter
+
         return self
 
     def clone(
@@ -304,13 +399,13 @@ class Y:
         show_context=None,
         show_time=None,
         show_delta=None,
-        line_length=None,
-        pair_delimiter=None,
-        enabled=None,
-        sort_dicts=None,
-        as_str=False,
         show_enter=None,
-        show_exit=None
+        show_exit=None,
+        sort_dicts=None,
+        enabled=None,
+        line_length=None,
+        context_delimiter=None,
+        pair_delimiter=None
     ):
         return Y(
             prefix=self.prefix if prefix is None else prefix,
@@ -319,13 +414,13 @@ class Y:
             show_context=self.show_context if show_context is None else show_context,
             show_time=self.show_time if show_time is None else show_time,
             show_delta=self.show_delta if show_delta is None else show_delta,
-            line_length=self.line_length if line_length is None else line_length,
-            pair_delimiter=self.pair_delimiter if pair_delimiter is None else pair_delimiter,
-            enabled=self.enabled if enabled is None else enabled,
-            sort_dicts=self.sort_dicts if sort_dicts is None else sort_dicts,
-            as_str=self.as_str if as_str is None else as_str,
             show_enter=self.show_enter if show_enter is None else show_enter,
             show_exit=self.show_exit if show_exit is None else show_exit,
+            sort_dicts=self.sort_dicts if sort_dicts is None else sort_dicts,
+            enabled=self.enabled if enabled is None else enabled,
+            line_length=self.line_length if line_length is None else line_length,
+            context_delimiter=self.context_delimiter if context_delimiter is None else context_delimiter,
+            pair_delimiter=self.pair_delimiter if pair_delimiter is None else pair_delimiter,
         )
 
     @contextmanager
@@ -336,13 +431,13 @@ class Y:
         show_context = self.show_context
         show_time = self.show_time
         show_delta = self.show_delta
-        line_length = self.line_length
-        pair_delimiter = self.pair_delimiter
-        enabled = self.enabled
-        sort_dicts = self.sort_dicts
-        as_str = self.as_str
         show_enter = (None,)
         show_exit = (None,)
+        sort_dicts = self.sort_dicts
+        enabled = self.enabled
+        line_length = self.line_length
+        context_delimiter = self.context_delimiter
+        pair_delimiter = self.pair_delimiter
         yield
         self.configure(
             prefix=prefix,
@@ -351,14 +446,23 @@ class Y:
             show_context=show_context,
             show_time=show_time,
             show_delta=show_delta,
-            line_length=line_length,
-            enabled=enabled,
-            sort_dicts=sort_dicts,
-            as_str=as_str,
             show_enter=show_enter,
             show_exit=show_exit,
+            sort_dicts=sort_dicts,
+            enabled=enabled,
+            context_delimiter=context_delimiter,
+            pair_delimiter=pair_delimiter,
+            line_length=line_length,
         )
 
+    @property
+    def delta(self):
+        return time.perf_counter() - self.start_time
+
+    @delta.setter
+    def delta(self, t):
+        self.start_time = time.perf_counter() + t
+
     def _format(self, call_frame, *args):
         prefix = call_or_value(self._prefix)
 
@@ -371,11 +475,11 @@ class Y:
         else:
             parts = []
         if self._show_time:
-            parts.append(f'@ {datetime.datetime.now().strftime("%H:%M:%S.%f")[:-3]}')
+            parts.append(f'@ {datetime.datetime.now().strftime("%H:%M:%S.%f")}')
 
         if self._show_delta:
-            t0 = (datetime.datetime.now() - starttime).total_seconds()
-            parts.append(f"\u0394 {t0:.3f}")
+            t0 = time.perf_counter() - self.start_time
+            parts.append(f"delta={t0:.6f}")
 
         context = " ".join(parts)
 
@@ -407,7 +511,7 @@ class Y:
         all_args_on_one_line = self._pair_delimiter.join(pairs_processed)
         multiline_args = len(all_args_on_one_line.splitlines()) > 1
 
-        context_delimiter = CONTEXT_DELIMITER if context else ""
+        context_delimiter = self._context_delimiter if context else ""
         all_pairs = prefix + context + context_delimiter + all_args_on_one_line
         first_line_too_long = len(all_pairs.splitlines()[0]) > self._line_length
 
@@ -440,6 +544,48 @@ class Y:
 
         return filename, line_number, parent_function
 
+    def __enter__(self):
+        if not hasattr(self, "cm_info"):
+            raise ValueError("not allowed as context_manager")
+        self.enter_time = time.perf_counter()
+        if self.show_enter:
+            if self.show_context:
+                parts = [self.cm_info]
+            else:
+                parts = []
+            if self.show_time:
+                parts.append(f'@ {datetime.datetime.now().strftime("%H:%M:%S.%f")}')
+
+            if self.show_delta:
+                t0 = time.perf_counter() - self.start_time
+                parts.append(f"delta={t0:.3f}")
+
+            context = " ".join(parts)
+            if context:
+                context += self.context_delimiter
+            do_output(self.output, f"{self.prefix() if callable(self.prefix) else self.prefix}{context}enter")
+        return self
+
+    def __exit__(self, *args):
+        if self.show_exit:
+            if self.show_context:
+                parts = [self.cm_info]
+            else:
+                parts = []
+            if self.show_time:
+                parts.append(f'@ {datetime.datetime.now().strftime("%H:%M:%S.%f")}')
+
+            if self.show_delta:
+                t0 = time.perf_counter() - self.start_time
+                parts.append(f"delta={t0:.3f}")
+
+            context = " ".join(parts)
+            if context:
+                context += self.context_delimiter
+            duration = time.perf_counter() - self.enter_time
+            do_output(self.output, f"{self._prefix() if callable(self.prefix) else self.prefix}{context}exit in {duration:.6f} seconds")
+            delattr(self, "cm_info")
+
 
 try:
     builtins = __import__("__builtin__")
@@ -462,8 +608,10 @@ def enable(value=None):
     return global_enabled
 
 
+set_defaults()
+
 ic = Y(prefix="ic| ")
-y = Y(prefix="y| ")
+y = Y()
 
 # source of asttokens.util
 
@@ -1462,8 +1610,6 @@ import sys as _sys
 import types as _types
 from io import StringIO as _StringIO
 
-__all__ = ["pprint", "pformat", "isreadable", "isrecursive", "saferepr", "PrettyPrinter", "pp"]
-
 
 def pprint(object, stream=None, indent=1, width=80, depth=None, *, compact=False, sort_dicts=True):
     """Pretty-print a Python object to a stream [default is sys.stdout]."""
